# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import copy, os, gettext, pickle
from gi.repository import Gtk, Gdk
from glogic import const
from glogic.Utils import *
from glogic.Components import *
from gettext import gettext as _

class CircuitManager():
	def __init__(self, parent, *args, **kwds):
		self.parent = parent
		self.components = []
		self.selected_components = []
		self.net_connections = []
		self.net_levels = []
		self.net_no_dot = []
		self.components_history = [[]]
		self.action_count = 0
		self.save_point = 0
		self.current_time = 0.0
		self.running_length = 0.0002
		self.need_save = False
		self.simple_change = True
		self.filepath = ""
		self.probe_levels_history = []
		self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
		
	def save_file(self, filepath):
		try:
			fp = open(filepath, mode="w", encoding="utf-8")
		except IOError:
			return True
		for c in self.components:
			if c[0] == const.component_net:
				fp.write("net\n position=%d,%d,%d,%d\n\n" % (c[1], c[2], c[3], c[4]))
			else:
				fp.write("%s\n" % c[0])
				fp.write(" position=%d,%d\n" % (c[1].pos_x, c[1].pos_y))
				fp.write(" input_level=%s\n" % ",".join([str(int(p)) for p in c[1].input_level]))
				fp.write(" output_level=%s\n" % ",".join([str(int(p)) for p in c[1].output_level]))
				fp.write(" properties=%s\n\n" % ",".join(["%s:%s" % (c[1].prop_names[i], encode_text(str(p))) for i, p in enumerate(c[1].values)]))
				
		fp.close()
		self.filepath = filepath
		self.need_save = False
		self.simple_change = True
		self.save_point = self.action_count
		self.parent.set_title("%s - %s" % (os.path.basename(filepath), const.app_name))
		return False
	
	def open_file(self, filepath):
		try:
			fp = open(filepath, mode="r", encoding="utf-8")
		except IOError:
			return True
		self.reset_circuit()
		component = None
		for l in fp:
			l = l.strip()
			if l == "net":
				component = [const.component_net, 0, 10, 10, 10]
			elif l in comp_dict:
				component = [l, copy.deepcopy(comp_dict[l])]
			elif component is not None:
				if l == "":
					self.components.append(component)
					component = None
					
				else:
					prop = l.split("=")
					if len(prop) == 2:
						if prop[0] == "position":
							p = prop[1].split(",")
							if component[0] == const.component_net:
								if len(p) == 4:
									component[1] = int(p[0])
									component[2] = int(p[1])
									component[3] = int(p[2])
									component[4] = int(p[3])
							else:
								if len(p) == 2:
									component[1].pos_x = int(p[0])
									component[1].pos_y = int(p[1])
						
						elif prop[0] == "input_level":
							if component[0] != const.component_net:
								p = prop[1].split(",")
								if len(component[1].input_level) == len(p):
									component[1].input_level = [bool(int(s if s.isdigit() else 0)) for s in p]
									
						elif prop[0] == "output_level":
							if component[0] != const.component_net:
								p = prop[1].split(",")
								if len(component[1].output_level) == len(p):
									component[1].output_level = [bool(int(s if s.isdigit() else 0)) for s in p]
									
						elif prop[0] == "properties":
							if component[0] != const.component_net:
								p = prop[1].split(",")
								p_pairs = [pp.split(":") for pp in p]
								i = 0
								for p in component[1].properties:
									if isinstance(p[1], tuple):
										if p[1][0] == const.property_select:
											for p_pair in p_pairs:
												if p_pair[0] == component[1].prop_names[i] and len(p_pair) == 2:
													component[1].values[i] = int(p_pair[1] if p_pair[1].isdigit() else 0)
													break
										elif p[1][0] == const.property_int:
											for p_pair in p_pairs:
												if p_pair[0] == component[1].prop_names[i] and len(p_pair) == 2:
													component[1].values[i] = int(p_pair[1] if p_pair[1].isdigit() else 0)
													break
										elif p[1][0] == const.property_float:
											try:
												for p_pair in p_pairs:
													if p_pair[0] == component[1].prop_names[i] and len(p_pair) == 2:
														component[1].values[i] = float(p_pair[1])
														break
											except ValueError:
												component[1].values[i] = 0.0
										else:
											for p_pair in p_pairs:
												if p_pair[0] == component[1].prop_names[i] and len(p_pair) == 2:
													component[1].values[i] = decode_text(p_pair[1])
													break
									elif p[1] == const.property_bool:
										for p_pair in p_pairs:
											if p_pair[0] == component[1].prop_names[i] and len(p_pair) == 2:
												component[1].values[i] = bool(int(p_pair[1] if p_pair[1].isdigit() else 0))
												break
									else:
										i -= 1
									i += 1
							component[1].propertyChanged(component[1].values)
			
		fp.close()
		self.components_history = [copy.deepcopy(self.components)]
		self.filepath = filepath
		self.parent.set_title("%s - %s" % (os.path.basename(filepath), const.app_name))
		return False
		
	def analyze_connections(self):
		# Analyze connections
		self.net_connections = []
		self.net_no_dot = []
		not_added = self.components[:]
		net_not_empty = True
		while True:
			connection = []
			changed_flag = True
			net_not_empty = False
			while changed_flag:
				changed_flag = False
				remlist = []
				for c in not_added:
					if c[0] == const.component_net:
						net_not_empty = True
						if len(connection) == 0:
							connection.append((c[1], c[2]))
							connection.append((c[3], c[4]))
							remlist.append(c)
						else:
							if (c[1], c[2]) in connection:
								if not (c[3], c[4]) in connection:
									connection.append((c[3], c[4]))
									changed_flag = True
								remlist.append(c)
							elif (c[3], c[4]) in connection:
								if not (c[1], c[2]) in connection:
									connection.append((c[1], c[2]))
									changed_flag = True
								remlist.append(c)
				for c in remlist:
					not_added.remove(c)
					
			if not net_not_empty:
				break
				
			for con_p in connection:
				cnt = 0
				for c in self.components:
					if c[0] == const.component_net:
						if con_p[0] == c[1] and con_p[1] == c[2]:
							cnt += 1
						elif con_p[0] == c[3] and con_p[1] == c[4]:
							cnt += 1
					else:
						for p in c[1].input_pins + c[1].output_pins:
							if con_p[0] == c[1].pos_x + p[0] and con_p[1] == c[1].pos_y + p[1]:
								cnt += 1
				if cnt <= 2:
					self.net_no_dot.append(con_p)
				
			self.net_connections.append(connection)
			
		for c in self.components:
			if c[0] != const.component_net:
				for p in c[1].input_pins + c[1].output_pins:
					found = False
					for net in self.net_connections:
						if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
							found = True
							break
					if not found:
						self.net_connections.append([(c[1].pos_x + p[0], c[1].pos_y + p[1])])
			
	def split_nets(self, x, y):
		# Split nets on the specified point
		spl_list = []
		for c in self.components:
			if c[0] == const.component_net:
				if (y - c[2]) * (c[3] - c[1]) == (x - c[1]) * (c[4] - c[2]):
					if (c[1] <= x <= c[3] and c[2] <= y <= c[4]) or (c[3] <= x <= c[1] and c[4] <= y <= c[2]):
						spl_list.append(c)
							
		for c in spl_list:
			self.components.remove(c)
			net_selected = False
			if c in self.selected_components:
				self.selected_components.remove(c)
				net_selected = True
			if x != c[1] or y != c[2]:
				if not [const.component_net, c[1], c[2], x, y] in self.components and not [const.component_net, x, y, c[1], c[2]] in self.components:
					component_data = [const.component_net, c[1], c[2], x, y]
					self.components.append(component_data)
					if net_selected:
						self.selected_components.append(component_data)
			if x != c[3] or y != c[4]:
				if not [const.component_net, c[3], c[4], x, y] in self.components and not [const.component_net, x, y, c[3], c[4]] in self.components:
					component_data = [const.component_net, x, y, c[3], c[4]]
					self.components.append(component_data)
					if net_selected:
						self.selected_components.append(component_data)
	
	def connect_nets(self, x, y, lock_selected = False):
		nets = []
		for c in self.components:
			if c[0] == const.component_net:
				if c[1] == x and c[2] == y or c[3] == x and c[4] == y:
					if not (lock_selected and c in self.selected_components):
						nets.append(c)
			else:
				# Cannot connect if there are terminals of components
				for p in c[1].input_pins + c[1].output_pins:
					if c[1].pos_x + p[0] == x and c[1].pos_y + p[1] == y:
						return
		
		if len(nets) == 2:
			if (nets[0][1] - nets[0][3]) * (nets[1][2] - nets[1][4]) == (nets[1][1] - nets[1][3]) * (nets[0][2] - nets[0][4]):
				if nets[0][1] == x and nets[0][2] == y:
					if nets[1][1] == x and nets[1][2] == y:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) <= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) <= 0:
							component_data = [const.component_net, nets[0][3], nets[0][4], nets[1][3], nets[1][4]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
					else:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) >= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) >= 0:
							component_data = [const.component_net, nets[0][3], nets[0][4], nets[1][1], nets[1][2]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
				else:
					if nets[1][1] == x and nets[1][2] == y:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) >= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) >= 0:
							component_data = [const.component_net, nets[0][1], nets[0][2], nets[1][3], nets[1][4]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
					else:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) <= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) <= 0:
							component_data = [const.component_net, nets[0][1], nets[0][2], nets[1][1], nets[1][2]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)

	def initialize_logic(self):
		self.probe_levels_history = []
		self.current_time = 0.0
	
	def set_netlevels(self):
		self.net_levels = [-1]*len(self.net_connections)
		for c in self.components:
			if c[0] != const.component_net:
				for i,p in enumerate(c[1].output_pins):
					for j,net in enumerate(self.net_connections):
						if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
							if self.net_levels[j] != -1 and self.net_levels[j] != c[1].output_level[i]:
								self.parent.statusbar.push(0, _("Output port is short circuit!"))
								return True
							self.net_levels[j] = c[1].output_level[i]

	def analyze_logic(self):
		counter = 0
		osc = False
		start_time = self.current_time
		stop_time = self.current_time + self.running_length
		for c in self.components:
			if c[0] != const.component_net:
				c[1].pre_output = [[0, l] for l in c[1].output_level]
				
		while start_time <= self.current_time < stop_time or self.current_time == -1:
			net_levels_history = []
			while True:
				self.set_netlevels()
			
				for c in self.components:
					if c[0] != const.component_net:
						input_datas = []
						for i,p in enumerate(c[1].input_pins):
							for j,net in enumerate(self.net_connections):
								if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
									if self.net_levels[j] == -1:
										self.parent.statusbar.push(0, _("Input port is open circuit!"))
										return True
									input_datas.append(self.net_levels[j])
									
						output_datas = c[1].calculate(input_datas)
						if output_datas is None:
							self.parent.statusbar.push(0, _("Input is undefined!"))
							return True
							
						if c[0] == const.component_OSC:
							if self.current_time == 0.0:
								c[1].tp_hl = c[1].values[1] * 0.000001 + c[1].values[0] * 0.0000005
								c[1].tp_lh = c[1].tp_hl
							else:
								c[1].tp_hl = c[1].values[0] * 0.0000005
								c[1].tp_lh = c[1].tp_hl
						
						for i, output in enumerate(output_datas):
							if c[1].pre_output[i][1] != output:
								if c[1].output_level[i] == output:
									c[1].pre_output[i][0] = 0
								elif output == True:
									if c[1].tp_lh != 0.0:
										c[1].pre_output[i][0] = self.current_time + c[1].tp_lh
									else:
										c[1].pre_output[i][0] = 0
										c[1].output_level[i] = output
								else:
									if c[1].tp_hl != 0.0:
										c[1].pre_output[i][0] = self.current_time + c[1].tp_hl
									else:
										c[1].pre_output[i][0] = 0
										c[1].output_level[i] = output
								c[1].pre_output[i][1] = output
							
				if self.net_levels in net_levels_history[:-1]:
					if self.net_levels == net_levels_history[len(net_levels_history) - 1]:
						self.parent.statusbar.push(0, "")
						break
					else:
						self.parent.statusbar.push(0, _("This circuit oscillates on infinite frequency!"))
						return True
							
				net_levels_history.append(self.net_levels)
							
				counter += 1
				if counter >= 10000:
					self.parent.statusbar.push(0, _("This logic is complexity!"))
					return True
					
			probe_levels = [self.current_time]
			for c in self.components:
				if c[0] == const.component_probe:
					probe_levels.append(c[1].input_level[0])
			self.probe_levels_history.append(probe_levels)
			
			tmptime = self.current_time
			flag = True
			for c in self.components:
				if c[0] != const.component_net:
					for output in c[1].pre_output:
						if output[0] != 0:
							if flag:
								self.current_time = output[0]
								flag = False
							elif self.current_time > output[0]:
								self.current_time = output[0]
			
			if tmptime == self.current_time:
				break
				
			for c in self.components:
				if c[0] != const.component_net:
					for i, output in enumerate(c[1].pre_output):
						if output[0] == self.current_time:
							c[1].output_level[i] = output[1]
							output[0] = 0
							
		return False

	def remove_selected_component(self):
		for c in self.selected_components:
			self.components.remove(c)
			
		for c in self.selected_components:
			if c[0] == const.component_net:
				self.connect_nets(c[1], c[2])
				self.connect_nets(c[3], c[4])
			else:
				for p in c[1].input_pins + c[1].output_pins:
					self.connect_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
					
		self.selected_components = []
		self.parent.prop_window.setComponent(None)
	
	def reset_circuit(self):
		self.selected_components = []
		self.parent.prop_window.setComponent(None)
		self.components = []
		self.components_history = [[]]
		self.action_count = 0
		self.save_point = 0
		self.need_save = False
		self.simple_change = True
		self.filepath = ""
		
	def push_history(self):
		self.action_count += 1
		if self.action_count < len(self.components_history):
			self.components_history = self.components_history[0:self.action_count]
			if self.action_count <= self.save_point:
				self.simple_change = False
		self.components_history.append(copy.deepcopy(self.components))
		self.need_save = True
		self.parent.set_title("%s [%s] - %s" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle, const.text_modified, const.app_name))
		
	def undo(self):
		self.action_count -= 1
		if self.action_count >= 0:
			self.components = copy.deepcopy(self.components_history[self.action_count])
			
		if self.save_point == self.action_count and self.simple_change:
			self.need_save = False
			self.parent.set_title("%s - %s" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle, const.app_name))
		else:
			self.need_save = True
			self.parent.set_title("%s [%s] - %s" % ((os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle), const.text_modified, const.app_name))
		
		self.selected_components = []
		self.parent.prop_window.setComponent(None)
		
	def redo(self):
		self.action_count += 1
		if self.action_count < len(self.components_history):
			self.components = copy.deepcopy(self.components_history[self.action_count])
			
		if self.save_point == self.action_count and self.simple_change:
			self.need_save = False
			self.parent.set_title("%s - %s" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle, const.app_name))
		else:
			self.need_save = True
			self.parent.set_title("%s [%s] - %s" % ((os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle), const.text_modified, const.app_name))
		
		self.selected_components = []
		self.parent.prop_window.setComponent(None)
	
	def copy(self):
		# FIXME copying and pasting doesn't work expectedly now.
		return
	
	def paste(self):
		return
	
	def get_circuit_rectangle(self):
		leftside = -1
		rightside = -1
		upperside = -1
		bottomside = -1
		
		for c in self.components:
			if c[0] == const.component_net:
				if leftside == -1 or c[1] < leftside:
					leftside = c[1]
				if rightside == -1 or rightside < c[1]:
					rightside = c[1]
				if upperside == -1 or c[2] < upperside:
					upperside = c[2]
				if bottomside == -1 or bottomside < c[2]:
					bottomside = c[2]
				if leftside == -1 or c[3] < leftside:
					leftside = c[3]
				if rightside == -1 or rightside < c[3]:
					rightside = c[3]
				if upperside == -1 or c[4] < upperside:
					upperside = c[4]
				if bottomside == -1 or bottomside < c[4]:
					bottomside = c[4]
			else:
				if leftside == -1 or c[1].comp_rect[0] + c[1].pos_x < leftside:
					leftside = c[1].comp_rect[0] + c[1].pos_x
				if upperside == -1 or c[1].comp_rect[1] + c[1].pos_y < upperside:
					upperside = c[1].comp_rect[1] + c[1].pos_y
				if rightside == -1 or rightside < c[1].comp_rect[2] + c[1].pos_x:
					rightside = c[1].comp_rect[2] + c[1].pos_x
				if bottomside == -1 or bottomside < c[1].comp_rect[3] + c[1].pos_y:
					bottomside = c[1].comp_rect[3] + c[1].pos_y
		
		if len(self.components) == 0:
			return (0, 0, 0, 0)
			
		return (leftside, upperside, rightside, bottomside)
