# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import math
from gettext import gettext as _
from glogic import const
from glogic.Components.SystemComponents import BaseComponent
from glogic.Utils import *

class NOT(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 90, 0]
		self.input_pins = [(10, -20)]
		self.output_pins = [(90, -20)]
		self.input_level = [False]
		self.output_level = [True]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
	
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
	
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -20), (30, -20))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (72, -20), (90, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -20), (30, -20))
			cairo_paths(cr, (72, -20), (90, -20))
		cairo_paths(cr, (30, -35), (30, -5), (60, -20))
		cr.close_path()
		cr.stroke()
		cr.arc(66, - 20, 6, 0, 2 * math.pi)
		cr.stroke()

	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 87 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False

	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 87 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False

	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		return [not input_datas[0]]

class AND(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 100, 0]
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -30), (30, -30))
			cr.stroke()
			if self.input_level[1]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (30, -10))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (80, -20), (100, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -30), (30, -30))
			cairo_paths(cr, (10, -10), (30, -10))
			cairo_paths(cr, (80, -20), (100, -20))
		cairo_paths(cr, (60, -40), (30, -40), (30, 0), (60, 0))
		cr.stroke()
		cr.arc(60, -20, 20, -0.5 * math.pi, 0.5 * math.pi)
		cr.stroke()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return [input_datas[0] and input_datas[1]]

class OR(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 100, 0]
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -30), (35, -30))
			cr.stroke()
			if self.input_level[1]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (35, -10))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (80, -20), (100, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -30), (35, -30))
			cairo_paths(cr, (10, -10), (35, -10))
			cairo_paths(cr, (80, -20), (100, -20))
		cairo_bezier(cr, 30, -40, 40, -30, 40, -10, 30, 0)
		cairo_bezier(cr, 30, -40, 50, -40, 65, -40, 80, -20)
		cairo_bezier(cr, 30, 0, 50, 0, 65, 0, 80, -20)
		cr.stroke()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return [input_datas[0] or input_datas[1]]

class XOR(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 100, 0]
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -30), (35, -30))
			cr.stroke()
			if self.input_level[1]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (35, -10))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (80, -20), (100, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -30), (35, -30))
			cairo_paths(cr, (10, -10), (35, -10))
			cairo_paths(cr, (80, -20), (100, -20))
		cairo_bezier(cr, 30, -40, 40, -30, 40, -10, 30, 0)
		cairo_bezier(cr, 25, -40, 35, -30, 35, -10, 25, 0)
		cairo_bezier(cr, 30, -40, 50, -40, 65, -40, 80, -20)
		cairo_bezier(cr, 30, 0, 50, 0, 65, 0, 80, -20)
		cr.stroke()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return [(input_datas[0] and not input_datas[1]) or (not input_datas[0] and input_datas[1])]

class NAND(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 110, 0]
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(110, -20)]
		self.input_level = [False, False]
		self.output_level = [True]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -30), (30, -30))
			cr.stroke()
			if self.input_level[1]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (30, -10))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (92, -20), (110, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -30), (30, -30))
			cairo_paths(cr, (10, -10), (30, -10))
			cairo_paths(cr, (92, -20), (110, -20))
		cairo_paths(cr, (60, -40), (30, -40), (30, 0), (60, 0))
		cr.stroke()
		cr.arc(60, -20, 20, -0.5 * math.pi, 0.5 * math.pi)
		cr.stroke()
		cr.arc(86, -20, 6, 0, 2 * math.pi)
		cr.stroke()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return [not (input_datas[0] and input_datas[1])]

class NOR(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 110, 0]
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(110, -20)]
		self.input_level = [False, False]
		self.output_level = [True]
		self.prop_names = ["tphl", "tplh"]
		self.properties.append((_("Propagation delay:"), None, ""))
		self.properties.append((_("tPHL:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("tPLH:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def propertyChanged(self, prop):
		self.tp_hl = prop[0] * 0.000001
		self.tp_lh = prop[1] * 0.000001
		return False
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -30), (35, -30))
			cr.stroke()
			if self.input_level[1]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (35, -10))
			cr.stroke()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (92, -20), (110, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -30), (35, -30))
			cairo_paths(cr, (10, -10), (35, -10))
			cairo_paths(cr, (92, -20), (110, -20))
		cairo_bezier(cr, 30, -40, 40, -30, 40, -10, 30, 0)
		cairo_bezier(cr, 30, -40, 50, -40, 65, -40, 80, -20)
		cairo_bezier(cr, 30, 0, 50, 0, 65, 0, 80, -20)
		cr.stroke()
		cr.arc(86, -20, 6, 0, 2 * math.pi)
		cr.stroke()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return [not (input_datas[0] or input_datas[1])]

class SW(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 70, 0]
		self.input_pins = []
		self.output_pins = [(70, -20)]
		self.input_level = []
		self.output_level = [False]
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (50, -20), (70, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
			if self.output_level[0]:
				cr.rectangle(30.5, -28.5, 13, 7)
			else:
				cr.rectangle(16.5, -28.5, 13, 7)
			cr.fill()
		else:
			cairo_paths(cr, (50, -20), (70, -20))
			cr.rectangle(17, -28, 12, 6)
		cr.rectangle(10, -40, 40, 40)
		cr.rectangle(15, -30, 30, 10)
		cr.stroke()
		cairo_draw_text(cr, layout, "L", 20, -10, 0.5, 0.5)
		cairo_draw_text(cr, layout, "H", 40, -10, 0.5, 0.5)
		cr.fill()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 67 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 67 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
		
	def click(self, x, y):
		self.output_level[0] = not self.output_level[0]
		return True

class VDD(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 30, 0]
		self.input_pins = []
		self.output_pins = [(20, 0)]
		self.input_level = []
		self.output_level = [True]
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			cr.set_source_rgb(1.0, 0.0, 0.0)
			cairo_paths(cr, (20, -20), (20, 0))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (20, -20), (20, 0))
		cairo_paths(cr, (10, -20), (30, -20))
		cr.rectangle(19, -21, 2, 2)
		cr.stroke()
		cairo_draw_text(cr, layout, "Vdd", 20, -30, 0.5, 0.5)
		cr.fill()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 27 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 27 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False

class GND(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -50, 30, 0]
		self.input_pins = []
		self.output_pins = [(20, -50)]
		self.input_level = []
		self.output_level = [False]
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (20, -50), (20, -30))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (20, -50), (20, -30))
		cairo_paths(cr, (10, -30), (30, -30))
		cairo_paths(cr, (16, -30), (12, -20))
		cairo_paths(cr, (22, -30), (18, -20))
		cairo_paths(cr, (28, -30), (24, -20))
		cr.stroke()
		cairo_draw_text(cr, layout, "GND", 20, -10, 0.5, 0.5)
		cr.fill()
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 27 and self.pos_y - 47 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 27 <= x2 and y1 <= self.pos_y - 47 and self.pos_y - 3 <= y2:
			return True
		return False

