# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import math, copy, cairo
from glogic import const
from glogic.Utils import cairo_paths
from glogic.Components import comp_dict
from gi.repository import Gtk, Gdk, GdkPixbuf, Pango, PangoCairo

class DrawArea(Gtk.ScrolledWindow):
	def __init__(self, parent):
		Gtk.ScrolledWindow.__init__(self)
		self.set_hexpand(True)
		self.set_vexpand(True)
		self.width = 1920
		self.height = 1080
		self.drawingarea = Gtk.DrawingArea()
		self.drawingarea.set_size_request(self.width, self.height)
		self.add_with_viewport(self.drawingarea)
		
		self.drawingarea.set_events(Gdk.EventMask.ENTER_NOTIFY_MASK |
		                            Gdk.EventMask.LEAVE_NOTIFY_MASK |
		                            Gdk.EventMask.POINTER_MOTION_MASK |
		                            Gdk.EventMask.BUTTON_PRESS_MASK |
		                            Gdk.EventMask.BUTTON_RELEASE_MASK)
		self.drawingarea.connect("draw", self.on_draw)
		self.drawingarea.connect("enter-notify-event", self.on_enter)
		self.drawingarea.connect("leave-notify-event", self.on_leave)
		self.drawingarea.connect("motion-notify-event", self.on_motion)
		self.drawingarea.connect("button-press-event", self.on_button_press)
		self.drawingarea.connect("button-release-event", self.on_button_release)
		
		self.cursor_x = 0
		self.cursor_y = 0
		self.parent = parent
		self.added = False
		self.preadd = False
		self.drag_enabled = False
		self.preselected_component = None
		self.component_dragged = False
		self.cursor_over = False
		self.nearest_component = None
		self.rect_select_enabled = False
		self.mouse_down = False
		self.mbitmap = None
		self.circuit = None
		self.redraw = True
		self.netstarted = False
		self.drawing_font = Pango.FontDescription("VL Gothic 10")
		self.mpixbuf = cairo.ImageSurface(cairo.FORMAT_RGB24, self.width, self.height)
		
	def on_draw(self, widget, cr):
			
		if self.redraw:
		
			mcr = cairo.Context(self.mpixbuf)
			if self.parent.running_mode:
				mcr.set_source_rgb(1.0, 1.0, 1.0)
			else:
				mcr.set_source_rgb(0.0, 0.0, 0.0)
				
			mcr.rectangle(0, 0, self.width, self.height)
			mcr.fill()
			
			mcr.translate(0.5, 0.5)
			mcr.set_line_width(1.0)
			layout = PangoCairo.create_layout(mcr)
			layout.set_font_description(self.drawing_font)
			
			if not self.parent.running_mode:
				# Draw grids
				mcr.set_source_rgb(0.15, 0.15, 0.15)
				for x in range(0, self.width, 10):
					cairo_paths(mcr, (x, 0), (x, self.height))
				for y in range(0, self.height, 10):
					cairo_paths(mcr, (0, y), (self.width, y))
				mcr.stroke()
			
			# Draw net
			for c in self.circuit.components:
				if c[0] == const.component_net:
					if not c in self.circuit.selected_components:
						if self.parent.running_mode:
							for i,net in enumerate(self.circuit.net_connections):
								if (c[1], c[2]) in net:
									if self.circuit.net_levels[i] == 1:
										mcr.set_source_rgb(1.0, 0.0, 0.0)
									elif self.circuit.net_levels[i] == 0:
										mcr.set_source_rgb(0.0, 0.0, 1.0)
									else:
										mcr.set_source_rgb(0.0, 0.0, 0.0)
						else:
							mcr.set_source_rgb(0.0, 0.0, 1.0)
						
						cairo_paths(mcr, (c[1], c[2]), (c[3], c[4]))
						mcr.stroke()
			
			# Draw component
			if self.parent.running_mode:
				mcr.set_source_rgb(0.0, 0.0, 0.0)
			else:
				mcr.set_source_rgb(0.0, 1.0, 0.0)
				
			for c in self.circuit.components:
				if c[0] != const.component_net:
					if not c in self.circuit.selected_components:
						mcr.translate(c[1].pos_x, c[1].pos_y)
						c[1].drawComponent(mcr, layout, self.parent.running_mode)
						mcr.translate(-c[1].pos_x, -c[1].pos_y)
			
			# Draw terminal of components
			if not self.parent.running_mode:
				mcr.set_source_rgb(1.0, 0.0, 0.0)
				for c in self.circuit.components:
					if c[0] != const.component_net:
						if not c in self.circuit.selected_components:
							for p in c[1].input_pins + c[1].output_pins:
								mcr.rectangle(c[1].pos_x+p[0]-1.5, c[1].pos_y+p[1]-1.5, 3, 3)
				mcr.fill()
			
			# Draw terminal of nets
			if self.parent.running_mode:
				mcr.set_source_rgb(0.0, 0.0, 0.0)
			else:
				mcr.set_source_rgb(1.0, 0.0, 0.0)
			
			for c in self.circuit.components:
				if c[0] == const.component_net:
					if not c in self.circuit.selected_components:
						if not self.parent.running_mode:
							mcr.rectangle(c[1]-1.5, c[2]-1.5, 3, 3)
							mcr.rectangle(c[3]-1.5, c[4]-1.5, 3, 3)
						elif not (c[1], c[2]) in self.circuit.net_no_dot:
							mcr.rectangle(c[1]-1.5, c[2]-1.5, 3, 3)
						elif not (c[3], c[4]) in self.circuit.net_no_dot:
							mcr.rectangle(c[3]-1.5, c[4]-1.5, 3, 3)
			mcr.fill()
			
			# Draw selected components
			if not self.parent.running_mode:
				if not self.component_dragged:
					mcr.set_source_rgb(1.0, 1.0, 1.0)
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							cairo_paths(mcr, (c[1], c[2]), (c[3], c[4]))
					mcr.stroke()
			
					for c in self.circuit.selected_components:
						if c[0] != const.component_net:
							mcr.translate(c[1].pos_x, c[1].pos_y)
							c[1].drawComponent(mcr, layout, self.parent.running_mode)
							mcr.translate(-c[1].pos_x, -c[1].pos_y)
							
					for c in self.circuit.selected_components:
						if c[0] != const.component_net:
							for p in c[1].input_pins + c[1].output_pins:
								mcr.rectangle(c[1].pos_x+p[0]-1.5, c[1].pos_y+p[1]-1.5, 3, 3)
							
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							mcr.rectangle(c[1]-1.5, c[2]-1.5, 3, 3)
							mcr.rectangle(c[3]-1.5, c[4]-1.5, 3, 3)
						
					mcr.fill()
			
		cr.set_source_surface(self.mpixbuf, 0, 0);
		cr.paint();
		
		cr.translate(0.5, 0.5)
		cr.set_line_width(1.0)
		layout = PangoCairo.create_layout(cr)
		layout.set_font_description(self.drawing_font)
		
		if self.rect_select_enabled:
			cr.set_source_rgba(1, 0.75, 0, 0.25)
			cr.rectangle(self.select_start_x-0.5, self.select_start_y-0.5, self.cursor_smooth_x - self.select_start_x, self.cursor_smooth_y- self.select_start_y)
			cr.fill()
			cr.set_source_rgb(1, 0.75, 0)
			cr.set_line_width(1.0)
			cr.rectangle(self.select_start_x-0.5, self.select_start_y-0.5, self.cursor_smooth_x - self.select_start_x, self.cursor_smooth_y- self.select_start_y)
			cr.stroke()
			cr.set_line_width(1.0)
			
		if not self.added and self.cursor_over and self.parent.comp_window.checked_button != const.component_none:
			if self.parent.comp_window.checked_button == const.component_net:
				# Draw cursor
				cr.set_source_rgb(1.0, 1.0, 1.0)
				cr.arc(self.cursor_x, self.cursor_y, 3, 0, 2 * math.pi)
				cr.stroke()
				
				if self.netstarted:
					# Draw net
					cr.set_source_rgb(0.0, 0.0, 1.0)
					if self.net_right and self.netstart_x < self.cursor_x or self.net_left and self.cursor_x < self.netstart_x:
						cairo_paths(cr, (self.netstart_x, self.netstart_y), (self.cursor_x, self.netstart_y))
						if self.netstart_y != self.cursor_y:
							cairo_paths(cr, (self.cursor_x, self.netstart_y), (self.cursor_x, self.cursor_y))
						cr.stroke()
						
						cr.set_source_rgb(1.0, 0.0, 1.0)
						cr.rectangle(self.cursor_x-1.5, self.netstart_y-1.5, 3, 3)
						cr.fill()
						
					else:
						if self.netstart_y != self.cursor_y:
							cairo_paths(cr, (self.netstart_x, self.netstart_y), (self.netstart_x, self.cursor_y))
						if self.netstart_x != self.cursor_x:
							cairo_paths(cr, (self.netstart_x, self.cursor_y), (self.cursor_x, self.cursor_y))
						cr.stroke()
						
						cr.set_source_rgb(1.0, 0.0, 1.0)
						cr.rectangle(self.netstart_x-1.5, self.cursor_y-1.5, 3, 3)
						cr.fill()
						
					cr.rectangle(self.netstart_x-1.5, self.netstart_y-1.5, 3, 3)
					cr.fill()
				
			else:
				# Draw picked component
				if self.preadd:
					cr.set_source_rgb(1.0, 0.75, 0.0)
				else:
					cr.set_source_rgb(1.0, 0.5, 0.0)
					
				logicand = self.pushed_component
				
				cr.translate(self.cursor_x, self.cursor_y)
				logicand.drawComponent(cr, layout, self.parent.running_mode)
				cr.translate(-self.cursor_x, -self.cursor_y)
			
		
		# Draw selected components
		if not self.parent.running_mode:
			if self.component_dragged:
				cr.set_source_rgb(1.0, 1.0, 1.0)
				for c in self.circuit.selected_components:
					if c[0] == const.component_net:
						cairo_paths(cr, (c[1], c[2]), (c[3], c[4]))
						cr.stroke()
			
				for c in self.circuit.selected_components:
					if c[0] != const.component_net:
						cr.translate(c[1].pos_x, c[1].pos_y)
						c[1].drawComponent(cr, layout, self.parent.running_mode)
						cr.translate(-c[1].pos_x, -c[1].pos_y)
						
				for c in self.circuit.selected_components:
					if c[0] != const.component_net:
						for p in c[1].input_pins + c[1].output_pins:
							cr.rectangle(c[1].pos_x+p[0]-1.5, c[1].pos_y+p[1]-1.5, 3, 3)
							
				for c in self.circuit.selected_components:
					if c[0] == const.component_net:
						cr.rectangle(c[1]-1.5, c[2]-1.5, 3, 3)
						cr.rectangle(c[3]-1.5, c[4]-1.5, 3, 3)
						
				cr.fill()
				
		
		# Highlight component
		if self.nearest_component is not None and not self.nearest_component in self.circuit.selected_components and not self.parent.running_mode:
			c = self.nearest_component
			if c[0] == const.component_net:
				cr.set_source_rgb(0.5, 0.5, 1.0)
				cairo_paths(cr, (c[1], c[2]), (c[3], c[4]))
				cr.stroke()
			else:
				cr.set_source_rgb(0.5, 1.0, 0.5)
				cr.translate(c[1].pos_x, c[1].pos_y)
				c[1].drawComponent(cr, layout, self.parent.running_mode)
				cr.translate(-c[1].pos_x, -c[1].pos_y)
		
		self.redraw = False
		self.added = False
	
	def on_leave(self, widget, event):
		self.cursor_over = False
		self.parent.nearest_component = None
		self.queue_draw()
	
	def on_enter(self, widget, event):
		self.cursor_over = True
	
	def on_motion(self, widget, event):
		
		oldcursor_x = self.cursor_x
		oldcursor_y = self.cursor_y
		
		if self.parent.comp_window.checked_button != const.component_none:
			# snap cursor to terminals
			min_dist = 225
			if self.parent.comp_window.checked_button == const.component_net and not event.state & Gdk.ModifierType.CONTROL_MASK:
				for c in self.circuit.components:
					if c[0] == const.component_net:
						if min_dist > (event.x - c[1]) ** 2 + (event.y - c[2]) ** 2:
							min_dist = (event.x - c[1]) ** 2 + (event.y - c[2]) ** 2
							self.cursor_x = c[1]
							self.cursor_y = c[2]
						if min_dist > (event.x - c[3]) ** 2 + (event.y - c[4]) ** 2:
							min_dist = (event.x - c[3]) ** 2 + (event.y - c[4]) ** 2
							self.cursor_x = c[3]
							self.cursor_y = c[4]
						if c[1] == c[3]:
							if c[2] < event.y < c[4] or c[4] < event.y < c[2]:
								if min_dist > (event.x - c[1]) ** 2:
									min_dist = (event.x - c[1]) ** 2
									self.cursor_x = c[1] - c[1] % 10
									self.cursor_y = event.y - event.y % 10
						if c[2] == c[4]:
							if c[1] < event.x < c[3] or c[3] < event.x < c[1]:
								if  min_dist > (event.y - c[2]) ** 2:
									min_dist = (event.y - c[2]) ** 2
									self.cursor_x = event.x - event.x % 10
									self.cursor_y = c[2] - c[2] % 10
							
					else:
						for p in c[1].input_pins + c[1].output_pins:
							if min_dist > (event.x - c[1].pos_x - p[0]) ** 2 + (event.y - c[1].pos_y - p[1]) ** 2:
								min_dist = (event.x - c[1].pos_x - p[0]) ** 2 + (event.y - c[1].pos_y - p[1]) ** 2
								self.cursor_x = c[1].pos_x + p[0]
								self.cursor_y = c[1].pos_y + p[1]
			
			if min_dist == 225:
				self.cursor_x = event.x - event.x % 10
				self.cursor_y = event.y - event.y % 10
			if oldcursor_x != self.cursor_x or oldcursor_y != self.cursor_y:
				self.queue_draw()
				
		else:
			if self.mouse_down and self.drag_enabled:
				# Move components
				ddx = event.x - self.cursor_smooth_x
				ddy = event.y - self.cursor_smooth_y
				old_delta_x = self.drag_delta_x
				old_delta_y = self.drag_delta_y
				if ddx >= 0:
					self.drag_delta_x = ddx - ddx % 10
				else:
					self.drag_delta_x = ddx + (-ddx % 10)
				if ddy >= 0:
					self.drag_delta_y = ddy - ddy % 10
				else:
					self.drag_delta_y = ddy + (-ddy % 10)
				if old_delta_x != self.drag_delta_x or old_delta_y != self.drag_delta_y:
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							c[1] += self.drag_delta_x - old_delta_x
							c[2] += self.drag_delta_y - old_delta_y
							c[3] += self.drag_delta_x - old_delta_x
							c[4] += self.drag_delta_y - old_delta_y
							if old_delta_x == 0 and old_delta_y == 0:
								self.circuit.connect_nets(c[1] - (self.drag_delta_x - old_delta_x), c[2] - (self.drag_delta_y - old_delta_y), lock_selected = True)
								self.circuit.connect_nets(c[3] - (self.drag_delta_x - old_delta_x), c[4] - (self.drag_delta_y - old_delta_y), lock_selected = True)
						else:
							c[1].pos_x += self.drag_delta_x - old_delta_x
							c[1].pos_y += self.drag_delta_y - old_delta_y
							if old_delta_x == 0 and old_delta_y == 0:
								for p in c[1].input_pins + c[1].output_pins:
									self.circuit.connect_nets(c[1].pos_x - (self.drag_delta_x - old_delta_x) + p[0], c[1].pos_y - (self.drag_delta_y - old_delta_y) + p[1], lock_selected = True)
								
					if not self.component_dragged:
						self.redraw = True
						
					self.component_dragged = True
					self.queue_draw()
					
			else:
				self.cursor_smooth_x = event.x
				self.cursor_smooth_y = event.y
				
				if self.rect_select_enabled:
					self.queue_draw()
				else:
					# Highlight component
					old_nearest_component = self.nearest_component
					self.nearest_component = None
					for c in self.circuit.components:
						if c[0] == const.component_net:
							if (c[1] - 3 <= self.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.cursor_smooth_y <= c[2] + 3):
								if ((c[4] - c[2]) * (self.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
									self.nearest_component = c
									break
						else:
							if c[1].isMouseOvered(self.cursor_smooth_x, self.cursor_smooth_y):
								self.nearest_component = c
								break
								
					if old_nearest_component is not self.nearest_component:
						self.queue_draw()
	
	def on_button_press(self, widget, event):
		
		if event.button == 1:
		
			if not self.parent.running_mode:
				if self.parent.comp_window.checked_button == const.component_none:
					# Check selected area
					self.rect_select_enabled = True
					for c in self.circuit.components:
						if c[0] == const.component_net:
							if (c[1] - 3 <= self.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.cursor_smooth_y <= c[2] + 3):
								if ((c[4] - c[2]) * (self.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
									if c in self.circuit.selected_components:
										self.drag_delta_x = 0
										self.drag_delta_y = 0
										self.drag_enabled = True
								
									self.preselected_component = c
									self.rect_select_enabled = False
									break
						else:
							if c[1].isMouseOvered(self.cursor_smooth_x, self.cursor_smooth_y):
								if c in self.circuit.selected_components:
									self.drag_delta_x = 0
									self.drag_delta_y = 0
									self.drag_enabled = True
							
								self.preselected_component = c
								self.rect_select_enabled = False
								break
							
					if self.rect_select_enabled:
						self.select_start_x = self.cursor_smooth_x
						self.select_start_y = self.cursor_smooth_y
				
				else:
					self.preadd = True
			else:
				for c in self.circuit.components:
					if c[0] != const.component_net:
						c[1].mouse_down(event.x, event.y)
			self.mouse_down = True
			self.cursor_smooth_x = event.x
			self.cursor_smooth_y = event.y
			self.queue_draw()
	
	def on_button_release(self, widget, event):
	
		if event.button == 1: # Left button released
		
			if not self.parent.running_mode:
				self.drag_enabled = False
				if self.component_dragged:
					self.component_dragged = False
					tmp_components = self.circuit.components[:]
					for c in tmp_components:
						if c[0] == const.component_net:
							self.circuit.split_nets(c[1], c[2])
							self.circuit.split_nets(c[3], c[4])
						else:
							for p in c[1].input_pins + c[1].output_pins:
								self.circuit.split_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
				
					tmp_components = self.circuit.components[:]
					for c in tmp_components:
						if c[0] == const.component_net:
							self.circuit.connect_nets(c[1], c[2])
							self.circuit.connect_nets(c[3], c[4])
							
					self.preselected_component = None
					self.circuit.push_history()
					self.parent.action_undo.set_sensitive(True)
					self.parent.action_redo.set_sensitive(False)
			
				elif self.parent.comp_window.checked_button == const.component_none:
					# Select component
					selected = False
					if self.preselected_component is not None:
						c = self.preselected_component
						if c[0] == const.component_net:
							if (c[1] - 3 <= self.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.cursor_smooth_y <= c[2] + 3):
								if ((c[4] - c[2]) * (self.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
									if event.state & Gdk.ModifierType.CONTROL_MASK:
										if not c in self.circuit.selected_components:
											self.circuit.selected_components.append(c)
										else:
											self.circuit.selected_components.remove(c)
									else:
										self.circuit.selected_components = [c]
									selected = True
								
						elif c[1].isMouseOvered(self.cursor_smooth_x, self.cursor_smooth_y):
							if event.state & Gdk.ModifierType.CONTROL_MASK:
								if not c in self.circuit.selected_components:
									self.circuit.selected_components.append(c)
								else:
									self.circuit.selected_components.remove(c)
							else:
								self.circuit.selected_components = [c]
							selected = True
							
						self.preselected_component = None
				
					elif self.rect_select_enabled:
						self.rect_select_enabled = False
						rectselected_components = []
						if self.select_start_x < self.cursor_smooth_x:
							rect_left = self.select_start_x
							rect_right = self.cursor_smooth_x
						else:
							rect_left = self.cursor_smooth_x
							rect_right = self.select_start_x
						if self.select_start_y < self.cursor_smooth_y:
							rect_top = self.select_start_y
							rect_bottom = self.cursor_smooth_y
						else:
							rect_top = self.cursor_smooth_y
							rect_bottom = self.select_start_y
						
						for c in self.circuit.components:
							if c[0] == const.component_net:
								if rect_left <= c[1] <= rect_right and rect_left <= c[3] <= rect_right and rect_top <= c[2] <= rect_bottom and rect_top <= c[4] <= rect_bottom:
									rectselected_components.append(c)
									selected = True
							elif c[1].isInRectangle(rect_left, rect_top, rect_right, rect_bottom):
									rectselected_components.append(c)
									selected = True
								
						if event.state & Gdk.ModifierType.CONTROL_MASK:
							for c in rectselected_components:
								if not c in self.circuit.selected_components:
									self.circuit.selected_components.append(c)
								
						else:
							self.circuit.selected_components = rectselected_components
				
					if not selected and not event.state & Gdk.ModifierType.CONTROL_MASK:
						self.circuit.selected_components = []
				
					self.set_selected_component_to_prop_window()
					
					if len(self.circuit.selected_components) == 0:
						self.parent.action_cut.set_sensitive(False)
						self.parent.action_copy.set_sensitive(False)
						self.parent.action_delete.set_sensitive(False)
					else:
						self.parent.action_cut.set_sensitive(True)
						self.parent.action_copy.set_sensitive(True)
						self.parent.action_delete.set_sensitive(True)
				
				elif self.parent.comp_window.checked_button == const.component_net:
					if self.netstarted:
						# Add net
						if self.net_right and self.netstart_x < self.cursor_x or self.net_left and self.cursor_x < self.netstart_x:
							component_data = [const.component_net, self.netstart_x, self.netstart_y, self.cursor_x, self.netstart_y]
							component_data2 = [const.component_net, self.cursor_x, self.netstart_y, self.netstart_x, self.netstart_y]
							if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
								self.circuit.components.append(component_data)
							
							if self.netstart_y != self.cursor_y:
								component_data = [const.component_net, self.cursor_x, self.netstart_y, self.cursor_x, self.cursor_y]
								component_data2 = [const.component_net, self.cursor_x, self.cursor_y, self.cursor_x, self.netstart_y]
								if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
									self.circuit.components.append(component_data)
					
						else:
							if self.netstart_y != self.cursor_y:
								component_data = [const.component_net, self.netstart_x, self.netstart_y, self.netstart_x, self.cursor_y]
								component_data2 = [const.component_net, self.netstart_x, self.cursor_y, self.netstart_x, self.netstart_y]
								if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
									self.circuit.components.append(component_data)
							
							if self.netstart_x != self.cursor_x:
								component_data = [const.component_net, self.netstart_x, self.cursor_y, self.cursor_x, self.cursor_y]
								component_data2 = [const.component_net, self.cursor_x, self.cursor_y, self.netstart_x, self.cursor_y]
								if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
									self.circuit.components.append(component_data)
					
						tmp_components = self.circuit.components[:]
						for c in tmp_components:
							if c[0] == const.component_net:
								self.circuit.split_nets(c[1], c[2])
								self.circuit.split_nets(c[3], c[4])
							else:
								for p in c[1].input_pins + c[1].output_pins:
									self.circuit.split_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
								
						tmp_components = self.circuit.components[:]
						for c in tmp_components:
							if c[0] == const.component_net:
								self.circuit.connect_nets(c[1], c[2])
								self.circuit.connect_nets(c[3], c[4])
							
						self.circuit.push_history()
						self.parent.action_undo.set_sensitive(True)
						self.parent.action_redo.set_sensitive(False)
					
					# Begin to create net
					self.netstart_x = self.cursor_x
					self.netstart_y = self.cursor_y
					self.netstarted = True
					self.net_left = True
					self.net_right = True
					for c in self.circuit.components:
						if c[0] == const.component_net:
							if c[2] == c[4] == self.netstart_y:
								if c[1] <= self.netstart_x <= c[3]:
									if c[1] != self.netstart_x:
										self.net_left = False
									if c[3] != self.netstart_x:
										self.net_right = False
								elif c[3] <= self.netstart_x <= c[1]:
									if c[3] != self.netstart_x:
										self.net_left = False
									if c[1] != self.netstart_x:
										self.net_right = False
									
							if self.net_left == self.net_right == False:
								break
							
				else:
					# Add component
					exist = False
					for c in self.circuit.components:
						if c[0] != const.component_net:
							if c[1].pos_x == self.cursor_x and c[1].pos_y == self.cursor_y:
								exist = True
								break
							
					if not exist:
						component_data = [self.parent.comp_window.checked_button, copy.deepcopy(comp_dict[self.parent.comp_window.checked_button])]
						component_data[1].pos_x = self.cursor_x
						component_data[1].pos_y = self.cursor_y
					
						if component_data[1].isInRectangle(0, 0, 1920, 1080):
							for p in component_data[1].input_pins + component_data[1].output_pins:
								self.circuit.split_nets(p[0] + self.cursor_x, p[1] + self.cursor_y)
					
							self.circuit.components.append(component_data)
							self.circuit.push_history()
							self.parent.action_undo.set_sensitive(True)
							self.parent.action_redo.set_sensitive(False)
					
				self.preadd = False
				self.added = True
				self.mouse_down = False
				self.redraw = True
				self.queue_draw()
			
			else:
				for c in self.circuit.components:
					if c[0] != const.component_net:
						if c[1].mouse_up(event.x, event.y):
							if not self.circuit.analyze_logic():
								self.parent.chart_window.chartarea.createChart()
							self.redraw = True
							self.queue_draw()
							break

		elif event.button == 3: # Right button released
	
			if not self.parent.running_mode:
				if self.parent.comp_window.checked_button == const.component_none:
					self.set_selected_component_to_prop_window()
					self.parent.prop_window.show_all()
					self.parent.action_property.set_active(True)
				elif not self.netstarted:
					self.parent.comp_window.uncheck_all_buttons()
					self.parent.action_net.set_active(False)
					self.parent.comp_window.checked_button = const.component_none
					self.pushed_component = comp_dict[const.component_none]
				else:
					# Finish creating net
					self.netstarted = False
				
			self.queue_draw()
			
	def set_selected_component_to_prop_window(self):
		if len(self.circuit.selected_components) == 1:
			if self.circuit.selected_components[0][0] != const.component_net:
				self.parent.prop_window.setComponent(self.circuit.selected_components[0][1])
			else:
				self.parent.prop_window.setComponent(None)
		else:
			self.parent.prop_window.setComponent(None)
